@@ -161,6 +161,10 @@ namespace xcas {
     do_maple_mode=512; Xcas_Style->value("Python ^==xor");
   }
 
+  static void cb_Xcas_Set_Python4(Fl_Menu_*, void*) {
+    do_maple_mode=1024; Xcas_Style->value("MicroPython");
+  }
+
   static void cb_Xcas_Set_Maple(Fl_Menu_*, void*) {
     do_maple_mode=1; Xcas_Style->value("Maple");
   }
@@ -177,6 +181,7 @@ namespace xcas {
     {gettext("Xcas"), 0,  (Fl_Callback*)cb_Xcas_Set_CPP, 0, 0, 0, 0, 14, 56},
     {gettext("Python ^==**"), 0,  (Fl_Callback*)cb_Xcas_Set_Python1, 0, 0, 0, 0, 14, 56},
     {gettext("Python ^==xor"), 0,  (Fl_Callback*)cb_Xcas_Set_Python2, 0, 0, 0, 0, 14, 56},
+    {gettext("MicroPython"), 0,  (Fl_Callback*)cb_Xcas_Set_Python4, 0, 0, 0, 0, 14, 56},
     {gettext("Maple"), 0,  (Fl_Callback*)cb_Xcas_Set_Maple, 0, 0, 0, 0, 14, 56},
     {gettext("Mupad"), 0,  (Fl_Callback*)cb_Xcas_Set_Mupad, 0, 0, 0, 0, 14, 56},
     {gettext("TI89/92"), 0,  (Fl_Callback*)cb_Xcas_Set_TI, 0, 0, 0, 0, 14, 56},
@@ -494,6 +499,7 @@ or default eval level)"));
     case 0: Xcas_Style->value("Xcas"); break;
     case 256: Xcas_Style->value("Python ^=**"); break;
     case 512: Xcas_Style->value("Python ^==xor"); break;
+    case 1024: Xcas_Style->value("MicroPython"); break;
     case 1: Xcas_Style->value("Maple"); break;
     case 2: Xcas_Style->value("Mupad"); break;
     case 3: Xcas_Style->value("Ti"); break;
@@ -605,10 +611,10 @@ or default eval level)"));
     giac::gnuplot_zmax=Xcas_Zmax->value();
     giac::gnuplot_tmin=Xcas_Tmin->value();
     giac::gnuplot_tmax=Xcas_Tmax->value();
-    xcas::Xcas_config.window_xmin=Xcas_WXmin->value();
-    xcas::Xcas_config.window_xmax=Xcas_WXmax->value();
-    xcas::Xcas_config.window_ymin=Xcas_WYmin->value();
-    xcas::Xcas_config.window_ymax=Xcas_WYmax->value();
+    giac::global_window_xmin=xcas::Xcas_config.window_xmin=Xcas_WXmin->value();
+    giac::global_window_xmax=xcas::Xcas_config.window_xmax=Xcas_WXmax->value();
+    giac::global_window_ymin=xcas::Xcas_config.window_ymin=Xcas_WYmin->value();
+    giac::global_window_ymax=xcas::Xcas_config.window_ymax=Xcas_WYmax->value();
     xcas::Xcas_config.ortho=Xcas_orthonormal->value();
     xcas::Xcas_config.autoscale=Xcas_autoscale->value();
     giac::class_minimum=Xcas_Class_min->value();
@@ -845,6 +851,10 @@ or default eval level)"));
     Xcas_Zmax->value(giac::gnuplot_zmax);
     Xcas_Tmin->value(giac::gnuplot_tmin);
     Xcas_Tmax->value(giac::gnuplot_tmax);
+    xcas::Xcas_config.window_xmin=global_window_xmin;
+    xcas::Xcas_config.window_xmax=global_window_xmax;
+    xcas::Xcas_config.window_ymin=global_window_ymin;
+    xcas::Xcas_config.window_ymax=global_window_ymax;
     Xcas_WXmin->value(xcas::Xcas_config.window_xmin);
     Xcas_WXmax->value(xcas::Xcas_config.window_xmax);
     Xcas_WYmin->value(xcas::Xcas_config.window_ymin);
