@@ -2,6 +2,10 @@
 #include "Editeur.h"
 #include "Input.h"
 #include "Tableur.h"
+#ifdef HAVE_LIBMICROPYTHON
+extern "C" int mp_token(const char * line);
+#endif
+
 /*
  *  Copyright (C) 2000,2014 B. Parisse, Institut Fourier, 38402 St Martin d'Heres
  *
@@ -67,7 +71,7 @@ namespace xcas {
     { FL_BLUE,       FL_COURIER_BOLD,   14 }  // G - Keywords
   };
   int styletable_n=sizeof(styletable_init) / sizeof(styletable_init[0]);
-  const char         *code_keywords[] = {   // List of known giac keywords...
+  const char *code_keywords[] = {   // List of known giac keywords...
     "BEGIN",
     "BREAK",
     "CATCH",
@@ -182,6 +186,7 @@ namespace xcas {
     "xor"
   };
   
+  
 //
 // 'compare_keywords()' - Compare two keywords...
 //
@@ -208,15 +213,11 @@ namespace xcas {
 // 'style_parse()' - Parse text and produce style data.
 //
 
-  void
-  style_parse(const char *text,
-	      char       *style,
-	      int        length) {
-    char	     current;
-    int	     col;
-    int	     last;
-    char	     buf[255],
-      *bufptr;
+  void style_parse(const char *text,char *style,int length,int mode) {
+    char current;
+    int	col;
+    int	last;
+    char buf[255], *bufptr;
     const char *temp;
     
     for (current = *style, col = 0, last = 0; length > 0; length --, text ++) {
@@ -253,10 +254,14 @@ namespace xcas {
 	    *bufptr = '\0';
 	    
 	    bufptr = buf;
-	    
-	    if (bsearch(&bufptr, code_keywords,
-			       sizeof(code_keywords) / sizeof(code_keywords[0]),
-			       sizeof(code_keywords[0]), compare_keywords)) {
+	    bool test;
+	    if (mode==4)
+	      test=is_python_keyword(bufptr);
+	    else
+	      test=bsearch(&bufptr, code_keywords,
+			   sizeof(code_keywords) / sizeof(code_keywords[0]),
+			   sizeof(code_keywords[0]), compare_keywords);
+	    if (test) {
 	      current='A';
 	      while (text < temp) {
 		*style++ = 'G';
@@ -269,19 +274,27 @@ namespace xcas {
 	      length ++;
 	      last = 1;
 	      continue;
-	    } else if (giac::vector_completions_ptr() && binary_search(giac::vector_completions_ptr()->begin(),giac::vector_completions_ptr()->end(),bufptr,alpha_order)) {
-	      current='A';
-	      while (text < temp) {
-		*style++ = 'F';
-		text ++;
-		length --;
-		col ++;
+	    } else {
+	      bool test;
+	      if (mode==4){
+		int tok=mp_token(bufptr);
+		test=is_python_builtin(bufptr) || tok;
+	      }
+	      else
+		test=giac::vector_completions_ptr() && binary_search(giac::vector_completions_ptr()->begin(),giac::vector_completions_ptr()->end(),bufptr,alpha_order);
+	      if (test) {
+		current='A';
+		while (text < temp) {
+		  *style++ = 'F';
+		  text ++;
+		  length --;
+		  col ++;
+		}
+		text --;
+		length ++;
+		last = 1;
+		continue;
 	      }
-	      
-	      text --;
-	      length ++;
-	      last = 1;
-	      continue;
 	    }
 	  }
 	}
@@ -388,7 +401,9 @@ namespace xcas {
     //  printf("start = %d, end = %d, text = \"%s\", style = \"%s\"...\n",
     //         start, end, text, style);
     
-    style_parse(text, style, end - start);
+    context * contextptr=get_context((Xcas_Text_Editor *) cbArg);
+    int mode=python_compat(contextptr) & 4;
+    style_parse(text, style, end - start,mode);
     
     //  printf("new style = \"%s\"...\n", style);
     
@@ -405,7 +420,7 @@ namespace xcas {
       text  = textbuf->text_range(start, end);
       style = stylebuf->text_range(start, end);
       
-      style_parse(text, style, end - start);
+      style_parse(text, style, end - start,mode);
       
       stylebuf->replace(start, end, style);
       ((Fl_Text_Editor *)cbArg)->redisplay_range(start, end);
@@ -2524,6 +2539,8 @@ namespace xcas {
 
   Xcas_Text_Editor::Xcas_Text_Editor(int X, int Y, int W, int H, Fl_Text_Buffer *b,const char* l ):Fl_Text_Editor(X,Y,W,H,l){
     styletable=vector<Fl_Text_Display::Style_Table_Entry>(styletable_init,styletable_init+styletable_n);
+    styletable[0].color=Xcas_editor_color;
+    cursor_color(Xcas_editor_color);
     tableur=0;
     gchanged=true;
     labeltype(FL_NO_LABEL);
@@ -2538,7 +2555,9 @@ namespace xcas {
     
     stylebuf = new Fl_Text_Buffer(buffer()->length());
     
-    style_parse(text, style, buffer()->length());
+    context * contextptr=get_context(this);
+    int mode=python_compat(contextptr) & 4;
+    style_parse(text, style, buffer()->length(),mode);
     
     stylebuf->text(style);
     delete[] style;
@@ -2559,6 +2578,7 @@ namespace xcas {
     Fl_Text_Buffer * b = new Fl_Text_Buffer;
     editor=new Xcas_Text_Editor(x,y+L,w,h-L,b,l);
     editor->Fl_Text_Display::textsize(labelsize());
+    editor->Fl_Text_Display::linenumber_width(3*labelsize());
     editor->labelsize(labelsize());
     log = 0;
     if (logo){
@@ -2582,7 +2602,7 @@ namespace xcas {
       nxt_button = new Fl_Button(x+w/3,y,w/12,L);
       nxt_button->labelsize(labelsize());
       nxt_button->label(gettext("nxt"));
-      nxt_button->tooltip(gettext("Find next occurence (defined by Edit->Search)"));
+      nxt_button->tooltip(gettext("Find next occurrence (defined by Edit->Search)"));
       nxt_button->callback((Fl_Callback *) cb_Editeur_Next);
       if (parent()==window()){
 	exec_button = new Save_Focus_Button(x+w/3-w/12,y,w/12,L);
@@ -3002,9 +3022,20 @@ namespace xcas {
     show_insert_position();
   }
 
+  vector<aide> micropython_filter_help(const vector<aide> & v_orig){
+    vector<aide> v;
+    for (int i=0;i<v_orig.size();++i){
+      const char * ptr=v_orig[i].cmd_name.c_str();
+      if (giac::is_python_builtin(ptr) || giac::is_python_keyword(ptr) || mp_token(ptr))
+	v.push_back(v_orig[i]);
+    }
+    return v;
+  }
+
   void Xcas_Text_Editor::set_tooltip(){
     static string toolt;
     History_Pack * hp=get_history_pack(this);
+    giac::context * contextptr=get_context(this);
     int pos=insert_position();
     int wbeg=buffer()->line_start(pos);
     string s(buffer()->text_range(wbeg,pos));
@@ -3015,7 +3046,8 @@ namespace xcas {
 	break;
     }
     if (s.size()>1 && k<s.size()){
-      const aide & help=helpon(s,*giac::vector_aide_ptr(),giac::language(hp?hp->contextptr:0),giac::vector_aide_ptr()->size());
+      vector<aide> vs=(contextptr && (python_compat(contextptr) & 4))?micropython_filter_help(*giac::vector_aide_ptr()):*giac::vector_aide_ptr();
+      const aide & help=helpon(s,vs,giac::language(hp?hp->contextptr:0),vs.size());
       toolt=writehelp(help,giac::language(hp?hp->contextptr:0));
       toolt += '\n';
       const char * ch=gettext("No help");
@@ -3249,7 +3281,16 @@ namespace xcas {
 	return 1;
       }
       int change_focus=0;
-      if (!ed && Fl::event_text() && (Fl::event_text()[0]=='\r') && !Fl::event_state(FL_SHIFT | FL_CTRL | FL_ALT) ){
+      if (Fl::event_text() && Fl::event_text()[0]=='\r' && Fl::event_state(FL_CTRL)){
+	if (ed)
+	  ed->eval();
+	else
+	  do_callback();
+	return 1;
+      }
+      if (
+	  (!ed && Fl::event_text() && (Fl::event_text()[0]=='\r') && !Fl::event_state(FL_SHIFT | FL_ALT) )	  
+	  ){
 	if (tableur){
 	  do_callback();
 	  return 1;
@@ -3357,6 +3398,7 @@ namespace xcas {
       }
       return 1;
     }
+    char before_ch=buffer()->text()[giacmax(insert_position()-1,0)];
     int res=Fl_Text_Editor::handle(event);
     if (!ed && event==FL_PASTE)
       resize_nl_before(1);
@@ -3378,7 +3420,7 @@ namespace xcas {
 	if (hp)
 	  hp->modified(true);
       }
-      if (!tableur && Fl::event_key()==FL_BackSpace && python_compat(contextptr)){
+      if (!tableur && Fl::event_key()==FL_BackSpace && python_compat(contextptr) && !isalnum(before_ch)){
 	dedent();
       }
       if (tableur && tableur->editing && Fl::event_key()==FL_BackSpace && value().empty()){
