@@ -131,6 +131,7 @@ namespace giac {
   int invmod(int n,int modulo);
   unsigned invmod(unsigned a,int b);
   int invmod(longlong a,int b);
+  longlong invmodll(longlong a,longlong b);
 #ifdef INT128
   int invmod(int128_t a,int b);
   inline int smod(int128_t r,int m){
@@ -145,6 +146,7 @@ namespace giac {
   }
   int smod(int a,int b); // where b is assumed to be positive
   int smod(longlong a,int b); 
+  longlong smodll(longlong res,longlong m);
   int simplify(int & a,int & b);
 
   struct ref_mpz_t {
@@ -266,6 +268,7 @@ namespace giac {
     virtual gen divide (const gen & g,GIAC_CONTEXT) const;
     gen operator / (const gen & g) const;
     virtual gen substract (const gen & g,GIAC_CONTEXT) const;
+    gen subtract(const gen & g,GIAC_CONTEXT) const;
     virtual gen operator / (const real_object & g) const;
     gen operator - (const gen & g) const;
     virtual gen operator - (const real_object & g) const;
@@ -372,6 +375,7 @@ namespace giac {
     virtual real_interval operator * (const real_interval & g) const;
     virtual gen divide (const gen & g,GIAC_CONTEXT) const;
     virtual gen substract (const gen & g,GIAC_CONTEXT) const;
+    gen subtract(const gen & g,GIAC_CONTEXT) const;
     virtual gen operator - (const real_object & g) const;
     virtual real_interval operator - (const real_interval & g) const ;
     virtual gen operator -() const;
@@ -792,7 +796,7 @@ namespace giac {
     // print then convert to a malloc-ated wchar_t *
     void modify(int i) { *this =gen(i); };
     const char * dbgprint() const; 
-    void uncoerce() ;
+    void uncoerce(size_t s=128) ;
     gen conj(GIAC_CONTEXT) const;
     gen re(GIAC_CONTEXT) const ;
     gen im(GIAC_CONTEXT) const ;
@@ -885,8 +889,13 @@ namespace giac {
   struct alias_ref_fraction { ref_count_t ref_count; alias_gen num; alias_gen den; };
   struct alias_ref_complex {
     ref_count_t ref_count;
+#ifdef BIGENDIAN
+    alias_gen im,re;
+    int display;
+#else
     int display;
     alias_gen re,im;
+#endif
   };
 
   struct ref_vecteur {
@@ -935,8 +944,13 @@ namespace giac {
 
   struct ref_complex {
     volatile ref_count_t ref_count;
+#ifdef BIGENDIAN
+    gen im,re;
+    int display;
+#else
     int display;
     gen re,im;
+#endif
     ref_complex(const std::complex<double> & c):ref_count(1),display(0),re(real(c)),im(imag(c)) {}
     ref_complex(const gen & R,const gen & I):ref_count(1),display(0),re(R),im(I) {}
     ref_complex(const gen & R,const gen & I,int display_mode):ref_count(1),display(display_mode),re(R),im(I) {}
@@ -1016,6 +1030,7 @@ namespace giac {
   gen rdiv(const gen & a,const gen & b,GIAC_CONTEXT0); // rational division
   inline gen operator /(const gen & a,const gen & b){ return rdiv(a,b); };
   gen operator %(const gen & a,const gen & b); // for int only
+  bool is_multiple(const gen & a,const gen &b);
   // gen inv(const gen & a);
   gen inv(const gen & a,GIAC_CONTEXT);
   inline wchar_t * wprint(const gen & g,GIAC_CONTEXT){ return g.wprint(contextptr); }
@@ -1110,6 +1125,7 @@ namespace giac {
   // more advanced arithmetic
   gen gcd(const gen & A,const gen & B,GIAC_CONTEXT);
   gen gcd(const gen & A,const gen & B);
+  int iegcd(int a_,int b_,int &u,int & v);
   gen lcm(const gen & a,const gen & b);
   gen simplify(gen & n, gen & d);
   void egcd(const gen &a,const gen &b, gen & u,gen &v,gen &d );
@@ -1118,6 +1134,7 @@ namespace giac {
   gen fracmod(const gen & a_orig,const gen & modulo); // -> p/q=a mod modulo
   bool fracmod(const gen & a_orig,const gen & modulo,gen & res);
   bool in_fracmod(const gen &m,const gen & a,mpz_t & d,mpz_t & d1,mpz_t & absd1,mpz_t &u,mpz_t & u1,mpz_t & ur,mpz_t & q,mpz_t & r,mpz_t &sqrtm,mpz_t & tmp,gen & num,gen & den);
+  bool alloc_fracmod(const gen & a_orig,const gen & modulo,gen & res,mpz_t & d,mpz_t & d1,mpz_t & absd1,mpz_t &u,mpz_t & u1,mpz_t & ur,mpz_t & q,mpz_t & r,mpz_t &sqrtm,mpz_t & tmp);
   gen powmod(const gen &base,const gen & expo,const gen & modulo);
   gen isqrt(const gen & A);
   gen re(const gen & a,GIAC_CONTEXT);
@@ -1235,7 +1252,7 @@ namespace giac {
       return false;
     }
     virtual bool operator == (const gen_user & a) const { return (*this) == gen(a); }
-    // must redefine > AND <= since we do not have symetrical type arguments
+    // must redefine > AND <= since we do not have symmetrical type arguments
     virtual gen operator > (const gen &) const { return gensizeerr(gettext("> not redefined")); }
     virtual gen operator > (const gen_user & a) const { return superieur_strict(*this, gen(a),0); }
     virtual gen operator <= (const gen &) const { return gensizeerr(gettext("<= not redefined")); }
