@@ -19,7 +19,7 @@
  */
 using namespace std;
 #include <cmath>
-#if !defined GIAC_HAS_STO_38 && !defined NSPIRE && !defined FXCG && !defined POCKETCAS
+#if !defined GIAC_HAS_STO_38 && !defined NSPIRE && !defined FXCG 
 #include <fstream>
 #endif
 #include <string>
@@ -990,7 +990,7 @@ namespace giac {
 #endif
 
   gen identificateur::eval(int level,const gen & orig,const context * contextptr) {
-    if (!ref_count)
+    if (!ref_count && !contextptr)
       return orig;
     gen evaled;
     // cerr << "idnt::eval " << *this << " " << level << '\n';
