@@ -47,10 +47,22 @@ namespace giac {
   static bool risch_poly_part(const vecteur & e,int shift,const identificateur & x,const vecteur & v,const gen & allowed_lnarg,gen & prim,gen & lncoeff,gen & remains_to_integrate,GIAC_CONTEXT);
   static bool risch_desolve(const gen & f,const gen & g,const identificateur & x,const vecteur & v,gen & y,bool f_is_derivative,GIAC_CONTEXT);
 
+  static gen ln_exp(const gen & g,GIAC_CONTEXT){
+    if (g.is_symb_of_sommet(at_exp))
+      return g._SYMBptr->feuille;
+    return symbolic(at_ln,g);
+  }
+
   // returns true & the tower of extension if g is elementary 
   // false otherwise
   static bool risch_tower(const identificateur & x,gen &g, vecteur & v,GIAC_CONTEXT){
     g=tsimplify(pow2expln(g,x,contextptr),contextptr);
+    // ln(exp(...))-> ...
+    vector<const unary_function_ptr *> vu;
+    vu.push_back(at_ln); 
+    vector <gen_op_context> vv;
+    vv.push_back(ln_exp);
+    g=ratnormal(subst(g,vu,vv,true,contextptr));
     v=rlvarx(g,x);
     const_iterateur it=v.begin(),itend=v.end();
     for (;it!=itend;++it){
@@ -547,7 +559,7 @@ namespace giac {
     // int( ipnum/ipden*exp(a*x),x)
     int save=calc_mode(contextptr);
     calc_mode(0,contextptr);
-    prim += _integrate(gen(makevecteur(expax*r2sym(ipnum/ipden,l,contextptr),x),_SEQ__VECT),contextptr);
+    prim += _integrate(gen(makevecteur(expax*r2sym(ipnum,l,contextptr)/r2sym(ipden,l,contextptr),x),_SEQ__VECT),contextptr);
     calc_mode(save,contextptr);
     if (is_undef(prim)) return false;
     // Hermite reduction 
@@ -759,7 +771,7 @@ namespace giac {
 	// Logarithmic part: compute resultant of num - t * den
 	polynome p1(s);
 	polynome pres=rothstein_trager_resultant(tmp.num,tmp.den,vl,p1,contextptr);
-	// Factorization, should return 1st order factor independant of
+	// Factorization, should return 1st order factor independent of
 	// the tower variables
 	factorization vden;
 	gen extra_div=1;
@@ -814,7 +826,6 @@ namespace giac {
     return true;
   }
 
-
   static gen risch_lin(const gen & e_orig,const identificateur & x,gen & remains_to_integrate,GIAC_CONTEXT){
     vecteur v;
     gen e=e_orig;
@@ -887,7 +898,7 @@ namespace giac {
       remsum += rem;
     }
     res = res+risch_lin(remsum,x,remains_to_integrate,contextptr);
-    if (is_zero(res)){ // perhaps we should derive res and substract it from e_orig
+    if (is_zero(res)){ // perhaps we should derive res and subtract it from e_orig
       remains_to_integrate=e_orig;
     }
     else {
