@@ -263,6 +263,7 @@
  & at_add_language,
  & at_additionally,
  & at_adjoint_matrix,
+ & at_display,
  & at_affix,
  & at_alg,
  & at_algsubs,
@@ -312,6 +313,7 @@
  & at_autosimplify,
  & at_avance,
  & at_avgRC,
+ & at_axis,
  & at_back,
  & at_background,
  & at_backward,
@@ -378,6 +380,8 @@
  & at_center2interval,
  & at_centered_cube,
  & at_centered_tetrahedron,
+ & at_center,
+ & at_circle,
  & at_cfactor,
  & at_cfsolve,
  & at_changebase,
@@ -440,6 +444,7 @@
  & at_convertir,
  & at_convexhull,
  & at_coordinates,
+ & at_coordinates,
  & at_copy,
  & at_correlation,
  & at_cos,
@@ -456,6 +461,7 @@
  & at_courbe_polaire,
  & at_covariance,
  & at_covariance_correlation,
+ & at_cp,
  & at_cpartfrac,
  & at_crationalroot,
  & at_crayon,
@@ -509,6 +515,7 @@
  & at_diag,
  & at_diff,
  & at_dim,
+ & at_ls,
  & at_display,
  & at_disque,
  & at_disque_centre,
@@ -533,12 +540,15 @@
  & at_baisse_crayon,
  & at_draw_arc,
  & at_draw_circle,
+ & at_draw_filled_arc,
+ & at_draw_filled_circle,
  & at_draw_line,
  & at_set_pixel,
  & at_draw_polygon,
  & at_draw_rectangle,
  & at_draw_string,
  & at_droit,
+ & at_line,
  & at_droite_tangente,
  & at_dsolve,
  & at_dtype,
@@ -642,6 +652,7 @@
  & at_float,
  & at_float2rational,
  & at_floor,
+ & at_fmod,
  & at_fonction,
  & at_fonction_derivee,
  & at_fopen,
@@ -850,10 +861,9 @@
  & at_join,
  & at_jordan,
  & at_jump,
- & at_jusque,
  & at_ker,
  & at_kernel,
- & at_keyboard,
+ & at_keydown,
  & at_kill,
  & at_kolmogorovd,
  & at_kolmogorovt,
@@ -910,6 +920,7 @@
  & at_lname,
  & at_lncollect,
  & at_lnexpand,
+ & at_locate,
  & at_locus,
  & at_log10,
  & at_logarithmic_regression,
@@ -918,6 +929,7 @@
  & at_logistic_regression,
  & at_logistic_regression_plot,
  & at_lower,
+ & at_ls,
  & at_lsmod,
  & at_lsq,
  & at_lu,
@@ -954,13 +966,16 @@
  & at_midpoint,
  & at_min,
  & at_min_proba_time,
+ & at_mkdir,
  & at_mkisom,
  & at_mksa,
  & at_modgcd,
  & at_modgcd_cachesize,
  & at_modp,
  & at_mods,
+ & at_monotonic,
  & at_montre_tortue,
+ & at_more,
  & at_moustache,
  & at_moyal,
  & at_moyenne,
@@ -1020,6 +1035,7 @@
  & at_ord,
  & at_order_size,
  & at_ordinate,
+ & at_ordinate,
  & at_orthocenter,
  & at_orthogonal,
  & at_os_version,
@@ -1051,6 +1067,7 @@
  & at_baisse_crayon,
  & at_pencolor,
  & at_baisse_crayon,
+ & at_slope,
  & at_penup,
  & at_perimeter,
  & at_perimeterat,
@@ -1198,9 +1215,13 @@
  & at_rat_jordan_block,
  & at_rationalroot,
  & at_ratnormal,
+ & at_radius,
  & at_rdiv,
  & at_re,
  & at_read,
+ & at_read16,
+ & at_read32,
+ & at_read_nand,
  & at_readrgb,
  & at_readwav,
  & at_real,
@@ -1247,6 +1268,7 @@
  & at_rhs,
  & at_right,
  & at_right_triangle,
+ & at_rm,
  & at_rm_a_z,
  & at_rm_all_vars,
  & at_rmbreakpoint,
@@ -1271,6 +1293,7 @@
  & at_rpn,
  & at_rref,
  & at_rsolve,
+ & at_input,
  & at_saute,
  & at_sauve,
  & at_save_history,
@@ -1320,6 +1343,7 @@
  & at_size,
  & at_sizes,
  & at_skip,
+ & at_sleep,
  & at_slope,
  & at_slopeat,
  & at_slopeatraw,
@@ -1340,6 +1364,7 @@
  & at_sortd,
  & at_sorted,
  & at_soundsec,
+ & at_speed,
  & at_sphere,
  & at_spherical2rectangular,
  & at_spline,
@@ -1453,6 +1478,7 @@
  & at_uniformd_icdf,
  & at_unitV,
  & at_unquote,
+ & at_leve_crayon,
  & at_upper,
  & at_usimplify,
  & at_valuation,
@@ -1464,6 +1490,7 @@
  & at_vertices,
  & at_vertices_abc,
  & at_vertices_abca,
+ & at_speed,
  & at_vpotential,
  & at_warn_equal_in_prog,
  & at_watch,
@@ -1482,6 +1509,8 @@
  & at_wilcoxont,
  & at_with_sqrt,
  & at_write,
+ & at_write16,
+ & at_write32,
  & at_writergb,
  & at_writewav,
  & at_wz_certificate,
